@@ -42,39 +42,51 @@ using namespace Rcpp;
 namespace STK
 {
 
-/** facade design pattern.
+/* facade design pattern.
  * The ClusterLauncher allow to create the strategy for estimating a mixture model
  * with less effort
  **/
-ClusterLauncher::ClusterLauncher( SEXP model, SEXP nbCluster, SEXP modelNames, SEXP strategy, SEXP r_critName )
-                                : s4_model_(model)
+ClusterLauncher::ClusterLauncher( SEXP model, SEXP nbCluster, SEXP modelNames, SEXP strategy, SEXP critName )
+                                : IRunnerBase()
+                                , s4_model_(model)
                                 , s4_strategy_(strategy)
                                 , v_nbCluster_(nbCluster)
                                 , v_modelNames_(modelNames)
-                                , critName_(Rcpp::as<std::string>(r_critName))
+                                , critName_(Rcpp::as<std::string>(critName))
                                 , handler_()
                                 , manager_(handler_)
                                 , p_composer_(0)
-                                , idData_()
-                                , idMixtureModel_(Clust::unknown_mixture_)
-                                , isFreeProp_(false)
-    {}
+                                , isHeterogeneous_(false)
+{}
+/* facade design pattern.
+ * The ClusterLauncher allow to create the strategy for estimating a mixture model
+ * with less effort
+ **/
+ClusterLauncher::ClusterLauncher( SEXP model, SEXP nbCluster, SEXP strategy, SEXP critName )
+                                : IRunnerBase()
+                                , s4_model_(model)
+                                , s4_strategy_(strategy)
+                                , v_nbCluster_(nbCluster)
+                                , v_modelNames_()
+                                , critName_(Rcpp::as<std::string>(critName))
+                                , handler_()
+                                , manager_(handler_)
+                                , p_composer_(0)
+                                , isHeterogeneous_(true)
+{}
 /* destructor. */
 ClusterLauncher::~ClusterLauncher() { if (p_composer_) delete p_composer_;}
 
 /* run the estimation */
 bool ClusterLauncher::run()
 {
+  // compute the best model
+  Real initCriter = s4_model_.slot("criterion");
   Real criter;
-  criter = selectBestModel();
-  if (criter == Arithmetic<Real>::max() || !Arithmetic<Real>::isFinite(criter))
-    return false;
-  // copy common part
-  std::string idModelName;
-  handler_.getIdModel( idData_, idModelName);
-  idMixtureModel_ = Clust::stringToMixture(idModelName);
+  if (!isHeterogeneous_) { criter = selectSingleBestModel();}
+  else                   { criter = selectHeteroBestModel();}
+  // get common part
   s4_model_.slot("criterion")    = criter;
-  s4_model_.slot("modelName")    = mixtureToString(idMixtureModel_, isFreeProp_);
   s4_model_.slot("nbCluster")    = p_composer_->nbCluster();
   s4_model_.slot("lnLikelihood") = p_composer_->lnLikelihood();
   s4_model_.slot("nbFreeParameter")= p_composer_->nbFreeParameter();
@@ -88,28 +100,38 @@ bool ClusterLauncher::run()
     fi[i] = p_composer_->computeLnLikelihood(i);
     zi[i] += (1 - baseIdx);  // set base 1 for the class labels
   }
-  // get specific parameters
-  getParameters();
+  if (criter == initCriter || !Arithmetic<Real>::isFinite(criter)) return false;
   return true;
 }
 
 /* get the parameters */
-Real ClusterLauncher::selectBestModel()
+Real ClusterLauncher::selectSingleBestModel()
 {
+  std::string idDataBestModel;
+  // component
+  Rcpp::S4 s4_component = s4_model_.slot("component");
+
   // wrap data matrix with Rcpp and wrap Rcpp matrix with STK++ matrix
-  NumericMatrix m_data = s4_model_.slot("data");
+  NumericMatrix m_data = s4_component.slot("data");
   Real criter = s4_model_.slot("criterion");
   RcppMatrix<double> data(m_data);
 
-  int nbSample   = m_data.rows();
-  IMixtureComposer* p_current =0;
+  int nbSample   = s4_model_.slot("nbSample");
+  IMixtureComposer*  p_current =0;
   IMixtureCriterion* p_criterion =0;
 
   try
   {
-    // check if a model is free prop or not and add data to handler (no copy)
-    Array1D<bool> v_free;
-    for (int l= 0; l <v_modelNames_.size() ; ++l)
+    // create criterion
+    if (critName_ == "BIC") { p_criterion = new BICMixtureCriterion();}
+    if (critName_ == "AIC") { p_criterion = new AICMixtureCriterion();}
+    if (critName_ == "ICL") { p_criterion = new ICLMixtureCriterion();}
+
+    // start the estimation process, should end with the best model according to
+    // the criteria
+    ClusterFacade facade(p_current);
+    facade.createFullStrategy(s4_strategy_);
+    for (int l=0; l <v_modelNames_.size(); ++l)
     {
       std::string idData = "model" + typeToString<int>(l);
       std::string idModel(as<std::string>(v_modelNames_[l]));
@@ -118,45 +140,30 @@ Real ClusterLauncher::selectBestModel()
       bool freeProp;
       Clust::Mixture model = Clust::stringToMixture(idModel, freeProp);
       handler_.addData(m_data, idData, Clust::mixtureToString(model));
-      v_free.push_back(freeProp);
-    }
-    // create criterion
-    if (critName_ == "BIC") { p_criterion = new BICMixtureCriterion();}
-    if (critName_ == "AIC") { p_criterion = new AICMixtureCriterion();}
-    if (critName_ == "ICL") { p_criterion = new ICLMixtureCriterion();}
 
-    // start the estimation process, should end with the best model according to
-    // the criteria
-    p_composer_ = 0;
-    for (int k=0; k <v_nbCluster_.length(); ++k)
-    {
-      int K = v_nbCluster_[k];
-      for (int l=0; l <v_modelNames_.size(); ++l)
+      for (int k=0; k <v_nbCluster_.length(); ++k)
       {
+        int K = v_nbCluster_[k];
         // create composer
-        if (v_free[l]) { p_current = new MixtureComposer(nbSample, K);}
-        else           { p_current = new MixtureComposerFixedProp(nbSample, K);}
+        if (freeProp) { p_current = new MixtureComposer(nbSample, K);}
+        else          { p_current = new MixtureComposerFixedProp(nbSample, K);}
+
         // create current mixture and register it
         std::string idData = "model" + typeToString<int>(l);
         static_cast<MixtureComposer*>(p_current)->createMixture(manager_, idData);
 
-        // create facade and strategy
-        ClusterFacade facade(p_current);
-        facade.createFullStrategy(s4_strategy_);
         // run estimation and get results if possible
-        if (facade.run())
+        if (!facade.run()) { msg_error_ += facade.error();}
+        // compute criterion and update model if necessary
+        p_criterion->setModel(p_current);
+        p_criterion->run();
+        if (criter > p_criterion->value())
         {
-          // compute criterion and update model if necessary
-          p_criterion->setModel(p_current);
-          if (!p_criterion->run()) { delete p_current; p_current = 0;}
-          else if (criter > p_criterion->value())
-          {
-            if (p_composer_) { std::swap(p_current, p_composer_);}
-            else             { p_composer_ = p_current; p_current = 0;}
-            idData_     = idData;
-            isFreeProp_ = v_free[l];
-            criter = p_criterion->value();
-          }
+          if (p_composer_) { std::swap(p_current, p_composer_);}
+          else             { p_composer_ = p_current; p_current = 0;}
+          s4_component.slot("modelName") = idModel;
+          idDataBestModel = idData;
+          criter = p_criterion->value();
         }
         // release current composer
         if (p_current) { delete p_current; p_current = 0;}
@@ -164,8 +171,11 @@ Real ClusterLauncher::selectBestModel()
     }
     // release
     delete p_criterion;
+    // get specific parameters
+    getParameters(s4_component, idDataBestModel);
     return criter;
-  } catch (Exception const& e)
+  }
+  catch (Exception const& e)
   {
     if (p_current) delete p_current;
     if (p_criterion) delete p_criterion;
@@ -175,19 +185,103 @@ Real ClusterLauncher::selectBestModel()
   return Arithmetic<Real>::max();
 }
 
-void ClusterLauncher::getParameters()
+/* select best heterogeneous model */
+Real ClusterLauncher::selectHeteroBestModel()
 {
-  Clust::MixtureClass mixtClass = Clust::MixtureToMixtureClass(idMixtureModel_);
-  switch (mixtClass)
+  // list of the component
+  Rcpp::List s4_list =s4_model_.slot("ldata");
+  Real criter =s4_model_.slot("criterion");
+  int nbSample =s4_model_.slot("nbSample");
+  // main pointer
+  IMixtureComposer* p_current =0;
+  IMixtureCriterion* p_criterion =0;
+  try
+  {
+    // create facade and strategy
+    ClusterFacade facade(p_current);
+    facade.createFullStrategy(s4_strategy_);
+    bool sameProp = true;
+    // loop over the list of component and fil handler_
+    for (int l=0; l <s4_list.size(); ++l)
+    {
+      // component
+      Rcpp::S4 s4_component = s4_list[l];
+      NumericMatrix m_data = s4_component.slot("data");
+      // id of the data set and of the model
+      std::string idData  = "model" + typeToString<int>(l);
+      std::string idModel = s4_component.slot("modelName");
+       // register
+      bool freeMixture;
+      Clust::Mixture model = Clust::stringToMixture(idModel, freeMixture);
+      handler_.addData(m_data, idData, Clust::mixtureToString(model));
+      // if one of the model is free proportion, then we use free proportion
+      sameProp &= (!freeMixture);
+    }
+    // create criterion
+    if (critName_ == "BIC") { p_criterion = new BICMixtureCriterion();}
+    if (critName_ == "AIC") { p_criterion = new AICMixtureCriterion();}
+    if (critName_ == "ICL") { p_criterion = new ICLMixtureCriterion();}
+
+    for (int k=0; k <v_nbCluster_.length(); ++k)
+    {
+      int K = v_nbCluster_[k];
+      // create composer
+      if (!sameProp) { p_current = new MixtureComposer(nbSample, K);}
+      else           { p_current = new MixtureComposerFixedProp(nbSample, K);}
+      // create all mixtures
+      static_cast<MixtureComposer*>(p_current)->createMixtures(manager_);
+      // run estimation and get results if possible
+      if (!facade.run()) { msg_error_ += facade.error();}
+      // compute criterion and update model if necessary
+      p_criterion->setModel(p_current);
+      p_criterion->run();
+      if (criter > p_criterion->value())
+      {
+        if (p_composer_) { std::swap(p_current, p_composer_);}
+        else             { p_composer_ = p_current; p_current = 0;}
+        criter = p_criterion->value();
+      }
+      // release current composer
+      if (p_current) { delete p_current; p_current = 0;}
+    }
+    // release
+    delete p_criterion;
+    // get parameters
+    for (int l=0; l <s4_list.size(); ++l)
+    {
+      // component
+      Rcpp::S4 s4_component = s4_list[l];
+      // id of the data set and of the model
+      std::string idData  = "model" + typeToString<int>(l);
+      getParameters(s4_component, idData);
+    }
+    //
+    return criter;
+  }
+  catch (Exception const& e)
+  {
+    if (p_current) delete p_current;
+    if (p_criterion) delete p_criterion;
+    ::Rf_error(e.error().c_str()) ;
+  }
+  // failed
+  return Arithmetic<Real>::max();
+}
+
+void ClusterLauncher::getParameters(Rcpp::S4& s4_component, std::string const& idData)
+{
+  std::string rModelName = s4_component.slot("modelName");
+  bool freeProp;
+  switch (Clust::mixtureToMixtureClass(Clust::stringToMixture(rModelName, freeProp)))
   {
     case Clust::Gaussian_:
-      getDiagGaussianParameters();
+      getDiagGaussianParameters(s4_component, idData);
       break;
     case Clust::Gamma_:
-      getGammaParameters();
+      getGammaParameters(s4_component, idData);
       break;
     case Clust::Categorical_:
-      getCategoricalParameters();
+      getCategoricalParameters(s4_component, idData);
       break;
     default:
       break;
@@ -195,35 +289,36 @@ void ClusterLauncher::getParameters()
 }
 
 /* get the diagonal Gaussian parameters */
-void ClusterLauncher::getDiagGaussianParameters()
+void ClusterLauncher::getDiagGaussianParameters(Rcpp::S4& s4_component, std::string const& idData)
 {
   // get parameters
   Array2D<Real> params;
-  static_cast<MixtureComposer*>(p_composer_)->getParameters(manager_,idData_, params);
+  static_cast<MixtureComposer*>(p_composer_)->getParameters(manager_,idData, params);
   // get dimensions
   int K = params.sizeRows()/2, nbVariable = params.sizeCols();
   // get results
   Array2D<Real> mean(K, nbVariable), sigma(K, nbVariable);
   for (int k=0; k<K; ++k)
   {
-    mean.row(k)   = params.row(2*k);
+    mean.row(k)  = params.row(2*k);
     sigma.row(k) = params.row(2*k+1);
   }
   // save results in s4_model
-  s4_model_.slot("mean")   = wrap(mean);
-  s4_model_.slot("sigma") = wrap(sigma);
+  s4_component.slot("mean")  = wrap(mean);
+  s4_component.slot("sigma") = wrap(sigma);
   // get data
-  RcppMatrix<double> m_data;
-  static_cast<MixtureComposer*>(p_composer_)->getData(manager_, idData_, m_data);
-  s4_model_.slot("data") = (Rcpp::Matrix< RcppMatrix<double>::Rtype_>)m_data;
+  RcppMatrix<double> m_data =  manager_.getData<double>(idData);
+//  RcppMatrix<double> m_data;
+//  manager_.getData(idData_, m_data);
+  s4_component.slot("data") = (Rcpp::Matrix< RcppMatrix<double>::Rtype_>)m_data;
 }
 
 /* get the gamma parameters */
-void ClusterLauncher::getGammaParameters()
+void ClusterLauncher::getGammaParameters(Rcpp::S4& s4_component, std::string const& idData)
 {
   // get parameters
   Array2D<Real> params;
-  static_cast<MixtureComposer*>(p_composer_)->getParameters(manager_,idData_, params);
+  static_cast<MixtureComposer*>(p_composer_)->getParameters(manager_,idData, params);
   // get dimensions
   int K = params.sizeRows()/2, nbVariable = params.sizeCols();
   // get results
@@ -234,27 +329,26 @@ void ClusterLauncher::getGammaParameters()
     scale.row(k) = params.row(2*k+1);
   }
   // save results in s4_model
-  s4_model_.slot("shape") = wrap(shape);
-  s4_model_.slot("scale") = wrap(scale);
+  s4_component.slot("shape") = wrap(shape);
+  s4_component.slot("scale") = wrap(scale);
   // get data
-  RcppMatrix<double> m_data;
-  static_cast<MixtureComposer*>(p_composer_)->getData(manager_, idData_, m_data);
-  s4_model_.slot("data") = (Rcpp::Matrix< RcppMatrix<double>::Rtype_>)m_data;
+  RcppMatrix<double> m_data =  manager_.getData<double>(idData);
+  //manager_.getData(idData_, m_data);
+  s4_component.slot("data") = (Rcpp::Matrix< RcppMatrix<double>::Rtype_>)m_data;
 }
 
 /* get the diagonal Categorical parameters */
-void ClusterLauncher::getCategoricalParameters()
+void ClusterLauncher::getCategoricalParameters(Rcpp::S4& s4_component, std::string const& idData)
 {
   // get parameters
   Array2D<Real> params;
-  static_cast<MixtureComposer*>(p_composer_)->getParameters(manager_,idData_, params);
+  static_cast<MixtureComposer*>(p_composer_)->getParameters(manager_,idData, params);
   params.shift(0,0);
   // save results in s4_model
-  s4_model_.slot("plkj") = wrap(params);
+  s4_component.slot("plkj") = wrap(params);
   // get data
-  RcppMatrix<int> m_data;
-  static_cast<MixtureComposer*>(p_composer_)->getData(manager_, idData_, m_data);
-  s4_model_.slot("data") = (Rcpp::Matrix< RcppMatrix<int>::Rtype_>) m_data;
+  RcppMatrix<int> m_data =  manager_.getData<int>(idData);
+  s4_component.slot("data") = (Rcpp::Matrix< RcppMatrix<int>::Rtype_>) m_data;
 }
 
 /* select best model*/
